# Copyright Amethyst Reese
# Licensed under the MIT license

"""
Fuzzy subcommand matching for click
"""

from .__version__ import __version__
from .core import FuzzyCommandGroup

__author__ = "Amethyst Reese"

__all__ = [
    "FuzzyCommandGroup",
]
